var Dfe71000_sincronizacaoCTE_Controller = function(ScreenService, templateManager, ZhFilterPreferences, MessageService, Dfe71000_sincronizacaoCTE_Service) {
    
    var getActiveWidget = () => {
        return templateManager.container.getActiveWidget();
    };
    
    let erroslist = {DOWNLOADXMLCTE_ERRO: {}, CTE_RECUSA_SERV_DESAC: {}};
    
    this.cnpjOnChange = (widget) => {
        let inscricaoEstadual = widget.getField("CDINSCESTA");
        
        if(widget.currentRow.NRINSJURIEST) {
            inscricaoEstadual.dataSourceFilter.forEach( (data) => {
                if(data.name == 'CDEMPRESA')    data.value = widget.currentRow.CDEMPRESA;
                if(data.name == 'NRINSJURIEST') data.value = widget.currentRow.NRINSJURIEST;
            });
        }
        inscricaoEstadual.readOnly = !widget.currentRow.NRINSJURIEST;
        inscricaoEstadual.setValue('');
    };

    this.empresaOnChange = (widget) => {
        let cnpj = widget.getField("NRINSJURIEST");
        let cnpjFilter = cnpj.dataSourceFilter.filter(data => data.name === 'CDEMPRESA')[0];
        
        if(widget.currentRow.CDEMPRESA) cnpjFilter.value = widget.currentRow.CDEMPRESA; 
        
        cnpj.readOnly = !widget.currentRow.CDEMPRESA;
        cnpj.setValue('');
        this.cnpjOnChange(widget);
    };
    
    this.setDefaultFilter = (widget) => {
        ZhFilterPreferences.openFilterWidget(widget).then(response => {
            let widFilter = templateManager.container.getActiveWidget();
            let clearFilter = widFilter.actions.filter(action => action.name === 'clear')[0];
            
            clearFilter.shortcutKey = 'ctrl+backspace';
            clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
            widFilter.applyDefaultValue();
            
            //widFilter.currentRow.DTEMISSAO = {start: "01/01/2000", formattedStart: "01/01/2000", end: "31/12/2021", formattedEnd: "31/12/2021"}; //descomentar para testar.
            
            widFilter.reload();
        });
    };
    
    this.setFilterBeforeApply = (args) => {
        let filterRow = args.e.preference.FILTER_ROW;
        let dsFilter = args.owner.dataSourceFilter;
        let nrinsjurpesq = dsFilter.filter(data => data.name === 'NRINSJURPESQ')[0];
        let dtemissao = dsFilter.filter(data => data.name === 'DTEMISSAO')[0];
        let cdstatus = dsFilter.filter(data => data.name === 'CDSTATUS')[0];
        let nracessocte = dsFilter.filter(data => data.name === 'NRACESSOCTE')[0];
        
        nrinsjurpesq.value = filterRow.NRINSJURIEST;
        dtemissao.value = [filterRow.DTEMISSAO.formattedStart, filterRow.DTEMISSAO.formattedEnd];
        cdstatus.value = filterRow.CDSTATUS;
        args.e.preference.CUSTOM_FILTER_ROW = {};
        
        if(filterRow.NRACESSOCTE) {
            if(typeof(nracessocte) === 'undefined')
                dsFilter.push({name: "NRACESSOCTE", operator: "=", value: filterRow.NRACESSOCTE});
            else
                nracessocte.value = filterRow.NRACESSOCTE;
        }
        else if(nracessocte) {
            dsFilter.forEach( (data) => {
                if(data.name == 'NRACESSOCTE') dsFilter.splice(dsFilter.indexOf(data), 1);
            });
        }
    };
    
    this.requestError = (error) => {
        if(error && error.data.error) {
            if (typeof(error.data.error) === 'string') {
                if(error.data.error in erroslist)   MessageService.showMessageByCode(error.data.error);
                else                                ScreenService.errorNotification(error.data.error);
            }
            else {
                console.error(error);
                MessageService.showMessageByCode('ERRO_GERAL');
            }
        }
        else {
            console.error(error);
            MessageService.showMessageByCode('ERRO_GERAL');
        }
    };
    
    this.montaConfigCte = (args) => {
        Dfe71000_sincronizacaoCTE_Service.montaConfigCte(args.e.preference.FILTER_ROW).then({}, this.requestError);
    };
    
    this.sincronizarCte = (args) => {
        Dfe71000_sincronizacaoCTE_Service.sincronizarCte(args.owner.widget.appliedFilter.FILTER_ROW).then(resp => {
            
            if(resp.dataset.sincronizarCte[0].ERROS.length > 0){
                let erros = resp.dataset.sincronizarCte[0].ERROS;
                
                erros.forEach(erro => {
                    if(typeof(erro) == 'string'){
                        ScreenService.errorNotification(erro);
                    }else{
                        console.log("Error: ", erro);
                    }
                });
            }
            else if(resp.dataset.sincronizarCte[0])  {
                MessageService.showMessageByCode('SINC_CTE_SUCESSO');
                getActiveWidget().reload();
            }
            else MessageService.showMessageByCode('ERRO_GERAL');
        }, this.requestError);
    };
    
    this.gerarXMLCte = (widget) => {
        let ctesMarcados = widget.hasCheckedRows() ? widget.getCheckedRows() : [widget.currentRow];
        
        Dfe71000_sincronizacaoCTE_Service.gerarXMLCte(ctesMarcados).then(response => {
             window.open(
                 window.serviceUrls.filter(x => {
                     return x.package == 'df';
                 })[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]="+response.dataset.gerarXmlCteManifestacao[0]);
                 
                 if(ScreenService.isSwipeOpen()) ScreenService.goBack();
                 getActiveWidget().reload();
        }, this.requestError);
    };
    
    this.abrirPopupJustificativa = (widget) => {
        MessageService.confirmMessageByCode('CTEINFO_PREST_DESACORDO').then(() => {
            let popup = templateManager.container.getWidget('popupJustificativa');
            
            popup.currentRow.NMRAZSOCEMIT = widget.currentRow.NMRAZSOCEMIT;
            popup.currentRow.NRACESSOCTE = widget.currentRow.NRACESSOCTE;
            
            ScreenService.openPopup(popup);
        });
    };
    
    this.prestServDesacCte = (widgetPopup, widgetCTe) => {
        if(!widgetPopup.isValid() || widgetPopup.currentRow.JUSTIFICATIVA.trim().length < 15)
            MessageService.showMessageByCode('ERRO_JUSTIFICATIVA');
        else {
            let rowCTe = widgetCTe.currentRow;
            rowCTe.JUSTIFICATIVA = widgetPopup.currentRow.JUSTIFICATIVA;
            
            Dfe71000_sincronizacaoCTE_Service.prestServDesacCte(rowCTe).then(response => {
                if (response.dataset.prestServDesacCte.SUCCESS) {
                    widgetPopup.currentRow.JUSTIFICATIVA = null;
                    ScreenService.closePopup();
                    ScreenService.goBack();
                    widgetCTe.activate();
                    widgetCTe.reload();
                    MessageService.showMessageByCode('CTE_SUCCESS_SERV_DESAC');
                }
                else if (response.dataset.prestServDesacCte.ERROR) {
                    ScreenService.errorNotification(response.dataset.prestServDesacCte.ERROR);
                }
                else {
                    console.error(response);
                    MessageService.showMessageByCode('ERRO_GERAL');
                }
            }, this.requestError);
        }
        
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe71000_sincronizacaoCTE_Controller", Dfe71000_sincronizacaoCTE_Controller);
});